searchData={"items":[{"type":"module","doc":"A implementation of [RFC 6902](https://tools.ietf.org/html/rfc6902) in pure Elixir.\n\nThe patch can be a single change or a list of things that shall be changed. Therefore\na list or a single JSON patch can be provided. Every patch belongs to a certain operation\nwhich influences the usage.\n\nAccording to [RFC 6901](https://tools.ietf.org/html/rfc6901) escaping of `/` and `~` is done\nby using `~1` for `/` and `~0` for `~`.","title":"Jsonpatch","ref":"Jsonpatch.html"},{"type":"function","doc":"Apply a Jsonpatch or a list of Jsonpatches to a map or struct. The whole patch will not be applied\nwhen any path is invalid or any other error occured. When a list is provided, the operations are\napplied in the order as they appear in the list.\n\nAtoms are never garbage collected. Therefore, `Jsonpatch` works by default only with maps\nwhich used binary strings as key. This behaviour can be controlled via the `:keys` option.","title":"Jsonpatch.apply_patch/3","ref":"Jsonpatch.html#apply_patch/3"},{"type":"function","doc":"iex> patch = [\n    ...> %{op: \"add\", path: \"/age\", value: 33},\n    ...> %{op: \"replace\", path: \"/hobbies/0\", value: \"Elixir!\"},\n    ...> %{op: \"replace\", path: \"/married\", value: true},\n    ...> %{op: \"remove\", path: \"/hobbies/2\"},\n    ...> %{op: \"remove\", path: \"/hobbies/1\"},\n    ...> %{op: \"copy\", from: \"/name\", path: \"/surname\"},\n    ...> %{op: \"move\", from: \"/home\", path: \"/work\"},\n    ...> %{op: \"test\", path: \"/name\", value: \"Bob\"}\n    ...> ]\n    iex> target = %{\"name\" => \"Bob\", \"married\" => false, \"hobbies\" => [\"Sport\", \"Elixir\", \"Football\"], \"home\" => \"Berlin\"}\n    iex> Jsonpatch.apply_patch(patch, target)\n    {:ok, %{\"name\" => \"Bob\", \"married\" => true, \"hobbies\" => [\"Elixir!\"], \"age\" => 33, \"surname\" => \"Bob\", \"work\" => \"Berlin\"}}\n\n    iex> # Patch will not be applied if test fails. The target will not be changed.\n    iex> patch = [\n    ...> %{op: \"add\", path: \"/age\", value: 33},\n    ...> %{op: \"test\", path: \"/name\", value: \"Alice\"}\n    ...> ]\n    iex> target = %{\"name\" => \"Bob\", \"married\" => false, \"hobbies\" => [\"Sport\", \"Elixir\", \"Football\"], \"home\" => \"Berlin\"}\n    iex> Jsonpatch.apply_patch(patch, target)\n    {:error, %Jsonpatch.Error{patch: %{\"op\" => \"test\", \"path\" => \"/name\", \"value\" => \"Alice\"}, patch_index: 1, reason: {:test_failed, \"Expected value '\\\"Alice\\\"' at '/name'\"}}}\n\n    iex> # Patch will succeed, not applying invalid path operations.\n    iex> patch = [\n    ...> %{op: \"replace\", path: \"/name\", value: \"Alice\"},\n    ...> %{op: \"replace\", path: \"/age\", value: 42}\n    ...> ]\n    iex> target = %{\"name\" => \"Bob\"} # No age in target\n    iex> Jsonpatch.apply_patch(patch, target, ignore_invalid_paths: true)\n    {:ok, %{\"name\" => \"Alice\"}}","title":"Examples - Jsonpatch.apply_patch/3","ref":"Jsonpatch.html#apply_patch/3-examples"},{"type":"function","doc":"Apply a Jsonpatch or a list of Jsonpatches to a map or struct. In case of an error\nit will raise an exception. When a list is provided, the operations are applied in\nthe order as they appear in the list.\n\n(See Jsonpatch.apply_patch/2 for more details)","title":"Jsonpatch.apply_patch!/3","ref":"Jsonpatch.html#apply_patch!/3"},{"type":"function","doc":"Creates a patch from the difference of a source map to a destination map or list.","title":"Jsonpatch.diff/2","ref":"Jsonpatch.html#diff/2"},{"type":"function","doc":"iex> source = %{\"name\" => \"Bob\", \"married\" => false, \"hobbies\" => [\"Elixir\", \"Sport\", \"Football\"]}\n    iex> destination = %{\"name\" => \"Bob\", \"married\" => true, \"hobbies\" => [\"Elixir!\"], \"age\" => 33}\n    iex> Jsonpatch.diff(source, destination)\n    [\n      %{path: \"/married\", value: true, op: \"replace\"},\n      %{path: \"/hobbies/2\", op: \"remove\"},\n      %{path: \"/hobbies/1\", op: \"remove\"},\n      %{path: \"/hobbies/0\", value: \"Elixir!\", op: \"replace\"},\n      %{path: \"/age\", value: 33, op: \"add\"}\n    ]","title":"Examples - Jsonpatch.diff/2","ref":"Jsonpatch.html#diff/2-examples"},{"type":"type","doc":"A valid Jsonpatch operation by RFC 6902","title":"Jsonpatch.t/0","ref":"Jsonpatch.html#t:t/0"},{"type":"module","doc":"Describe an error that occured while patching.","title":"Jsonpatch.Error","ref":"Jsonpatch.Error.html"},{"type":"type","doc":"","title":"Jsonpatch.Error.t/0","ref":"Jsonpatch.Error.html#t:t/0"},{"type":"module","doc":"The add operation is the operation for creating/updating values.\nValues can be inserted in a list using an index or appended using a `-`.","title":"Jsonpatch.Operation.Add","ref":"Jsonpatch.Operation.Add.html"},{"type":"module","doc":"iex> add = %Add{path: \"/a/b\", value: 1}\n    iex> target = %{\"a\" => %{\"c\" => false}}\n    iex> Jsonpatch.Operation.Add.apply(add, target, [])\n    {:ok, %{\"a\" => %{\"b\" => 1, \"c\" => false}}}\n\n    iex> add = %Add{path: \"/a/1\", value: \"b\"}\n    iex> target = %{\"a\" => [\"a\", \"c\"]}\n    iex> Jsonpatch.Operation.Add.apply(add, target, [])\n    {:ok, %{\"a\" => [\"a\", \"b\", \"c\"]}}\n\n    iex> add = %Add{path: \"/a/-\", value: \"z\"}\n    iex> target = %{\"a\" => [\"x\", \"y\"]}\n    iex> Jsonpatch.Operation.Add.apply(add, target, [])\n    {:ok, %{\"a\" => [\"x\", \"y\", \"z\"]}}","title":"Examples - Jsonpatch.Operation.Add","ref":"Jsonpatch.Operation.Add.html#module-examples"},{"type":"function","doc":"","title":"Jsonpatch.Operation.Add.apply/3","ref":"Jsonpatch.Operation.Add.html#apply/3"},{"type":"type","doc":"","title":"Jsonpatch.Operation.Add.t/0","ref":"Jsonpatch.Operation.Add.html#t:t/0"},{"type":"module","doc":"Represents the handling of JSON patches with a copy operation.","title":"Jsonpatch.Operation.Copy","ref":"Jsonpatch.Operation.Copy.html"},{"type":"module","doc":"iex> copy = %Jsonpatch.Operation.Copy{from: \"/a/b\", path: \"/a/e\"}\n  iex> target = %{\"a\" => %{\"b\" => %{\"c\" => \"Bob\"}}, \"d\" => false}\n  iex> Jsonpatch.Operation.Copy.apply(copy, target, [])\n  {:ok, %{\"a\" => %{\"b\" => %{\"c\" => \"Bob\"}, \"e\" => %{\"c\" => \"Bob\"}}, \"d\" => false}}","title":"Examples - Jsonpatch.Operation.Copy","ref":"Jsonpatch.Operation.Copy.html#module-examples"},{"type":"function","doc":"","title":"Jsonpatch.Operation.Copy.apply/3","ref":"Jsonpatch.Operation.Copy.html#apply/3"},{"type":"type","doc":"","title":"Jsonpatch.Operation.Copy.t/0","ref":"Jsonpatch.Operation.Copy.html#t:t/0"},{"type":"module","doc":"Move operations change the position of values in map or struct.","title":"Jsonpatch.Operation.Move","ref":"Jsonpatch.Operation.Move.html"},{"type":"module","doc":"iex> move = %Jsonpatch.Operation.Move{from: \"/a/b\", path: \"/a/e\"}\n    iex> target = %{\"a\" => %{\"b\" => %{\"c\" => \"Bob\"}}, \"d\" => false}\n    iex> Jsonpatch.Operation.Move.apply(move, target, [])\n    {:ok, %{\"a\" => %{\"e\" => %{\"c\" => \"Bob\"}}, \"d\" => false}}","title":"Examples - Jsonpatch.Operation.Move","ref":"Jsonpatch.Operation.Move.html#module-examples"},{"type":"function","doc":"","title":"Jsonpatch.Operation.Move.apply/3","ref":"Jsonpatch.Operation.Move.html#apply/3"},{"type":"type","doc":"","title":"Jsonpatch.Operation.Move.t/0","ref":"Jsonpatch.Operation.Move.html#t:t/0"},{"type":"module","doc":"A JSON patch remove operation is responsible for removing values.","title":"Jsonpatch.Operation.Remove","ref":"Jsonpatch.Operation.Remove.html"},{"type":"module","doc":"iex> remove = %Jsonpatch.Operation.Remove{path: \"/a/b\"}\n    iex> target = %{\"a\" => %{\"b\" => %{\"c\" => \"Bob\"}}, \"d\" => false}\n    iex> Jsonpatch.Operation.Remove.apply(remove, target, [])\n    {:ok, %{\"a\" => %{}, \"d\" => false}}\n\n    iex> remove = %Jsonpatch.Operation.Remove{path: \"/a/b\"}\n    iex> target = %{\"a\" => %{\"b\" => nil}, \"d\" => false}\n    iex> Jsonpatch.Operation.Remove.apply(remove, target, [])\n    {:ok, %{\"a\" => %{}, \"d\" => false}}","title":"Examples - Jsonpatch.Operation.Remove","ref":"Jsonpatch.Operation.Remove.html#module-examples"},{"type":"function","doc":"","title":"Jsonpatch.Operation.Remove.apply/3","ref":"Jsonpatch.Operation.Remove.html#apply/3"},{"type":"type","doc":"","title":"Jsonpatch.Operation.Remove.t/0","ref":"Jsonpatch.Operation.Remove.html#t:t/0"},{"type":"module","doc":"The replace module helps replacing values in maps and structs by paths.","title":"Jsonpatch.Operation.Replace","ref":"Jsonpatch.Operation.Replace.html"},{"type":"module","doc":"iex> add = %Jsonpatch.Operation.Replace{path: \"/a/b\", value: 1}\n    iex> target = %{\"a\" => %{\"b\" => 2}}\n    iex> Jsonpatch.Operation.Replace.apply(add, target, [])\n    {:ok, %{\"a\" => %{\"b\" => 1}}}","title":"Examples - Jsonpatch.Operation.Replace","ref":"Jsonpatch.Operation.Replace.html#module-examples"},{"type":"function","doc":"","title":"Jsonpatch.Operation.Replace.apply/3","ref":"Jsonpatch.Operation.Replace.html#apply/3"},{"type":"type","doc":"","title":"Jsonpatch.Operation.Replace.t/0","ref":"Jsonpatch.Operation.Replace.html#t:t/0"},{"type":"module","doc":"A test operation in a JSON patch prevents the patch application or allows it.","title":"Jsonpatch.Operation.Test","ref":"Jsonpatch.Operation.Test.html"},{"type":"module","doc":"iex> test = %Jsonpatch.Operation.Test{path: \"/x/y\", value: \"Bob\"}\n  iex> target = %{\"x\" => %{\"y\" => \"Bob\"}}\n  iex> Jsonpatch.Operation.Test.apply(test, target, [])\n  {:ok, %{\"x\" => %{\"y\" => \"Bob\"}}}","title":"Examples - Jsonpatch.Operation.Test","ref":"Jsonpatch.Operation.Test.html#module-examples"},{"type":"function","doc":"","title":"Jsonpatch.Operation.Test.apply/3","ref":"Jsonpatch.Operation.Test.html#apply/3"},{"type":"type","doc":"","title":"Jsonpatch.Operation.Test.t/0","ref":"Jsonpatch.Operation.Test.html#t:t/0"},{"type":"module","doc":"Types","title":"Jsonpatch.Types","ref":"Jsonpatch.Types.html"},{"type":"type","doc":"","title":"Jsonpatch.Types.casted_array_index/0","ref":"Jsonpatch.Types.html#t:casted_array_index/0"},{"type":"type","doc":"","title":"Jsonpatch.Types.casted_fragment/0","ref":"Jsonpatch.Types.html#t:casted_fragment/0"},{"type":"type","doc":"","title":"Jsonpatch.Types.casted_object_key/0","ref":"Jsonpatch.Types.html#t:casted_object_key/0"},{"type":"type","doc":"","title":"Jsonpatch.Types.convert_fn/0","ref":"Jsonpatch.Types.html#t:convert_fn/0"},{"type":"type","doc":"","title":"Jsonpatch.Types.error/0","ref":"Jsonpatch.Types.html#t:error/0"},{"type":"type","doc":"","title":"Jsonpatch.Types.error_reason/0","ref":"Jsonpatch.Types.html#t:error_reason/0"},{"type":"type","doc":"","title":"Jsonpatch.Types.json_container/0","ref":"Jsonpatch.Types.html#t:json_container/0"},{"type":"type","doc":"Keys options:\n\n- `:strings` (default) - decodes path fragments as binary strings\n- `:atoms` - path fragments are converted to atoms\n- `:atoms!` - path fragments are converted to existing atoms\n- `{:custom, convert_fn}` - path fragments are converted with `convert_fn`","title":"Jsonpatch.Types.opt_keys/0","ref":"Jsonpatch.Types.html#t:opt_keys/0"},{"type":"type","doc":"Types options:\n\n- `:keys` - controls how path fragments are decoded.","title":"Jsonpatch.Types.opts/0","ref":"Jsonpatch.Types.html#t:opts/0"},{"type":"exception","doc":"JsonpatchException will be raised if a patch is applied with \"!\"\nand the patching fails.","title":"JsonpatchException","ref":"JsonpatchException.html"},{"type":"extras","doc":"# Jsonpatch\n\n![Elixir CI](https://github.com/corka149/jsonpatch/workflows/Elixir%20CI/badge.svg)\n[![Coverage Status](https://coveralls.io/repos/github/corka149/jsonpatch/badge.svg?branch=master)](https://coveralls.io/github/corka149/jsonpatch?branch=master)\n[![Generic badge](https://img.shields.io/badge/Mutation-Tested-success.svg)](https://shields.io/)\n[![Maintenance](https://img.shields.io/badge/Maintained%3F-yes-green.svg)](https://github.com/corka149/jsonpatch/graphs/commit-activity)\n[![Hex.pm Version](https://img.shields.io/hexpm/v/jsonpatch.svg?style=flat-square)](https://hex.pm/packages/jsonpatch)\n\nAn implementation of [RFC 6902](https://tools.ietf.org/html/rfc6902) in pure Elixir.\n\nFeatures:\n\n- Creating a patch by comparing to maps and lists\n- Apply patches to maps and lists - supports operations:\n    - add\n    - replace\n    - remove\n    - copy\n    - move\n    - test\n- Escaping of \"`/`\" (by \"`~1`\") and \"`~`\" (by \"`~0`\")\n- Allow usage of `-` for appending things to list (Add and Copy operation)","title":"Jsonpatch","ref":"readme.html"},{"type":"extras","doc":"#","title":"Getting started - Jsonpatch","ref":"readme.html#getting-started"},{"type":"extras","doc":"The package can be installed by adding `jsonpatch` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:jsonpatch, \"~> 2.2\"}\n  ]\nend\n```\n\nThe docs can be found at [https://hexdocs.pm/jsonpatch](https://hexdocs.pm/jsonpatch).\n\n#","title":"Installation - Jsonpatch","ref":"readme.html#installation"},{"type":"extras","doc":"```elixir\niex> source = %{\"name\" => \"Bob\", \"married\" => false, \"hobbies\" => [\"Sport\", \"Elixir\", \"Football\"]}\niex> destination = %{\"name\" => \"Bob\", \"married\" => true, \"hobbies\" => [\"Elixir!\"], \"age\" => 33}\niex> Jsonpatch.diff(source, destination)\n[\n  %{path: \"/married\", value: true, op: \"replace\"},\n  %{path: \"/hobbies/2\", op: \"remove\"},\n  %{path: \"/hobbies/1\", op: \"remove\"},\n  %{path: \"/hobbies/0\", value: \"Elixir!\", op: \"replace\"},\n  %{path: \"/age\", value: 33, op: \"add\"}\n]\n```\n\n#","title":"Create a diff - Jsonpatch","ref":"readme.html#create-a-diff"},{"type":"extras","doc":"```elixir\niex> patch = [\n  %{op: \"add\", path: \"/age\", value: 33},\n  %{op: \"replace\", path: \"/hobbies/0\", value: \"Elixir!\"},\n  %{op: \"replace\", path: \"/married\", value: true},\n  %{op: \"remove\", path: \"/hobbies/1\"},\n  %{op: \"remove\", path: \"/hobbies/2\"}\n]\niex> target = %{\"name\" => \"Bob\", \"married\" => false, \"hobbies\" => [\"Sport\", \"Elixir\", \"Football\"]}\niex> Jsonpatch.apply_patch(patch, target)\n{:ok, %{\"name\" => \"Bob\", \"married\" => true, \"hobbies\" => [\"Elixir!\"], \"age\" => 33}}\n```","title":"Apply patches - Jsonpatch","ref":"readme.html#apply-patches"},{"type":"extras","doc":"- [Official RFC 6902](https://tools.ietf.org/html/rfc6902)\n- [Inspiration: python-json-patch](https://github.com/stefankoegl/python-json-patch)","title":"Important sources - Jsonpatch","ref":"readme.html#important-sources"}],"content_type":"text/markdown"}